@@ -115,9 +115,9 @@ test('shows warning', () => {
   equal(warn.calls, [
     [
       'Browserslist: caniuse-lite is outdated. Please run:\n' +
-        '  npx browserslist@latest --update-db\n' +
+        '  npx update-browserslist-db@latest\n' +
         '  Why you should do it regularly: ' +
-        'https://github.com/browserslist/browserslist#browsers-data-updating'
+        'https://github.com/browserslist/update-db#readme'
     ]
   ])
 })
